#include "eaudio.h"

GenericPlugin *player_plugs = NULL;
GenericPlugin *analysis_plugs = NULL;

GenericPlugin *SearchPlugins(GenericPlugin *list, char *id)
{
   GenericPlugin *cur = list;
   while (cur) {
      if (!strcasecmp(id, cur->id))
         EDBUG_RETURN(cur);
      if (strstr(cur->id, id))
         EDBUG_RETURN(cur);
      cur = cur->next;
   }
   EDBUG_RETURN(NULL);
}

static int play_none(char *song)
{
   char s[4096];

   sprintf(s, "eAudio couldn't figure out what song type \"%s\" is..", song);
   if (Confirmation(s,
                    "Click Ok to edit the song's information and manually set the player plugin.")) {
      DrawInfoEditor(TRUE, cur_item);
      return -1;
   }
   return 0;
}

static void get_info_none(struct playlist_item *song)
{
   song->length = 0;
}

struct player_plugin *DummyPlayer(GenericPlugin *plug)
{
   struct player_plugin *cur;

   EDBUG(6, "DummyPlayer");
   cur = Emalloc(sizeof(struct player_plugin));

   cur->init = NULL;
   cur->config = NULL;
   cur->seek_to_frame = NULL;
   cur->play = play_none;
   cur->pause = NULL;
   cur->stop = NULL;
   cur->get_info = get_info_none;
   cur->streamable = FALSE;
   cur->parent = plug;
   EDBUG_RETURN(cur);
}

struct player_plugin *ReturnPlayerPlugin(char *search)
{
   GenericPlugin *temp;
   
   EDBUG(5, "ReturnPlayerPlugin");
   if (!player_plugs)
      EDBUG_RETURN(NULL);
   temp = SearchPlugins(player_plugs, search);
   if (temp)
      EDBUG_RETURN(temp->data);
   EDBUG_RETURN(NULL);
}

static void do_analysis_none(char *buffer, int size)
{
   usleep(300);
}


struct analysis_plugin *DummyAnalysis(GenericPlugin *plug)
{
   struct analysis_plugin *cur;
   cur = Emalloc(sizeof(struct analysis_plugin));
              
   EDBUG(6, "DummyAnalysis");
   cur->init = NULL;
   cur->do_analysis = do_analysis_none;
   cur->close = NULL;
   cur->config = NULL;
   cur->parent = plug;
   EDBUG_RETURN(cur);
}

struct analysis_plugin *loadAnalysisSyms(GenericPlugin *plug)
{
   struct analysis_plugin *temp = NULL;
   struct analysis_plugin *(*setup_syms) (void) = NULL;
   
   EDBUG(4, "loadAnalysisSyms");
   if (!plug)
      EDBUG_RETURN(NULL);
   g_module_symbol(plug->handle, "plugin_data", (gpointer) &setup_syms);
   if (setup_syms)
      temp = setup_syms();
   else {
      fprintf(stderr, "g_module_error: %s\n", g_module_error());
      EDBUG_RETURN(NULL);
   }
   if (!temp)
      EDBUG_RETURN(NULL);
   temp->parent = plug;
   EDBUG_RETURN(temp);
}

struct player_plugin *loadPlayerSyms(GenericPlugin *plug)
{
   struct player_plugin *temp = NULL;
   struct player_plugin *(*setup_syms) (void) = NULL;

   EDBUG(4, "loadPlayerSyms");
   if (!plug)
      EDBUG_RETURN(NULL);
   g_module_symbol(plug->handle, "plugin_data", (gpointer) &setup_syms);
   if (setup_syms)
      temp = setup_syms();
   else {
      fprintf(stderr, "g_module error: %s\n", g_module_error());
      EDBUG_RETURN(NULL);
   }
   if (!temp)
      EDBUG_RETURN(NULL);
   temp->init(temp);
   temp->parent = plug;
   EDBUG_RETURN(temp);
}

static int loadGenericSyms(GenericPlugin *plug)
{
   GenericPlugin *temp;
   GenericPlugin *(*setup_plug)(void) = NULL;
   
   EDBUG(4, "loadGenericSyms");
   if (!plug)
      EDBUG_RETURN(FALSE);
   g_module_symbol(plug->handle, "setup_plugin", (gpointer) &setup_plug);
   if (setup_plug)
      temp = setup_plug();
   else {
      fprintf(stderr, "g_module_error: %s\n", g_module_error());
      EDBUG_RETURN(FALSE);
   }
   if (!temp)
      EDBUG_RETURN(FALSE);
   plug->id = temp->id;
   plug->version = temp->version;
   plug->copyright = temp->copyright;
   plug->type = temp->type;
   Efree(temp);
   EDBUG_RETURN(TRUE);
}  

GenericPlugin *InitPlugins(int type, char *path)
{
   GenericPlugin *cur = NULL;
   GenericPlugin *first = NULL;
   char tempname[FILEPATH_LEN_MAX], *buffer;
   FILE *temp;
   int i = 0, j;
   
   EDBUG(4, "InitPlugins");
   cur = Emalloc(sizeof(GenericPlugin));
   first = cur;
   cur->id = duplicate("Dummy");
   cur->type = type;
   switch (type) {
      case TYPE_PLAYER: cur->data = DummyPlayer(cur); break;
      case TYPE_ANALYSIS: cur->data = DummyAnalysis(cur); break;
      default: break;
   }
   tmpnam(tempname);
   if (!(temp = fopen(tempname, "w"))) {
      fprintf(stderr, "Couldn't get a temp file in InitPlugins\n");
      EDBUG_RETURN_;
   } 
   j = strlen(path);
   if (j > 1)
      if (path[j - 1] == '/')
         path[j - 1] = '\0';
   i = dodirtolist(path, "*.so.0", temp, 0);
   fclose(temp);
   if (!i) {
      if (!(temp = fopen(tempname, "r"))) {
         fprintf(stderr, "Couldn't read temp file in InitPlugins\n");
         EDBUG_RETURN_;
      }     
      buffer = Ecalloc(FILEPATH_LEN_MAX, 1);
      cur->next = Emalloc(sizeof(GenericPlugin)); 
      while (GetNextLine(buffer, temp)) {
         cur->next->handle = g_module_open(buffer, G_MODULE_BIND_LAZY);
         if (!cur->next->handle) {
            fprintf(stderr, "g_module error: %s\n", g_module_error());
            continue;
         }
         j = loadGenericSyms(cur->next);
         if (j && (cur->next->type == type)) {  
            switch (type) {
               case TYPE_PLAYER: cur->next->data = loadPlayerSyms(cur->next);
                                 break;
               case TYPE_ANALYSIS: cur->next->data = loadAnalysisSyms(cur->next);
                                   break;
               default: break;
            }
            cur = cur->next;
            printf("\t%s %s, (C) %s\n", cur->id, cur->version, cur->copyright);
            cur->next = Emalloc(sizeof(GenericPlugin));
         }
      }
      Efree(buffer);
      fclose(temp);
   }
   if (cur->next)
      Efree(cur->next);
   cur->next = NULL;
   rm(tempname);
   printf("\n");
   EDBUG_RETURN(first);
}

void InitPlayers(char *path)
{
   EDBUG(4, "InitPlayers");
   printf("     Player Modules:\n");
   player_plugs = InitPlugins(TYPE_PLAYER, path);
   EDBUG_RETURN_;
}

struct analysis_plugin *ReturnAnalysisPlugin(char *search)
{
   GenericPlugin *temp;

   EDBUG(4, "ReturnAnalysisPlugin");
   if (!analysis_plugs)
      EDBUG_RETURN(NULL);
   temp = SearchPlugins(analysis_plugs, search);
   if (temp)
      EDBUG_RETURN(temp->data);
   EDBUG_RETURN(NULL);
}

void InitAnalysis(char *path)
{
   EDBUG(4, "InitAnalysis");
   printf("     Analysis Modules:\n");
   analysis_plugs = InitPlugins(TYPE_ANALYSIS, path);
   EDBUG_RETURN_;
}

int getnumtracks_cda(void)
{
   struct player_plugin *cd;
   int (*getnumtracks) (void);

   EDBUG(4, "getnumtracks_cda");
   cd = ReturnPlayerPlugin("cd");
   if (!cd)
      EDBUG_RETURN(0);
   g_module_symbol(cd->parent->handle, "getnumtracks", (gpointer) & getnumtracks);
   if (!getnumtracks)
      EDBUG_RETURN(0);
   EDBUG_RETURN(getnumtracks());
}
